#include "JParserObj.h"
#include "JInteger.h"
#include "JRealData.h"
#include "JIntegerData.h"
#include "JStringData.h"
#include "JIntegerProperty.h"
#include "JIntegerListProperty.h"
#include "JStringProperty.h"
#include "JLinkObj.h"
#include "JBytesData.h"


char* theJParserObj = JParserObj().Register();


void JParserObj::writeContent(JOutputStream& os) {
  JAddObj::writeContent(os);
  putObject(os, "format", format);
}

void JParserObj::readContent(JDictionary& dict) {
  JAddObj::readContent(dict);
  JObject* obj_format = getObject( dict, "format");
  if ( obj_format ) format = *(JString*)obj_format;
}

const char* JParserObj::className() const { return "JParserObj";}


JObject* JParserObj::clone() const { return new JParserObj(*this);}


// class constructor
JParserObj::JParserObj() {
  igm = 1;
  ogm = MAXVALS + MAXVALS + MAXVALS;
}


void JParserObj::draw(JGraphics g, int x, int y, int w, int h) {
  JRect rect(x, y, w, h);
  drawText(g, JString("Parser"), rect);
}


boolean JParserObj::inputAllowed(int n)
{ return !inputSet(n).last();}
JString JParserObj::inputTag(int n) {
  char* tag[] = {
    "inString" };
  return( tag[n] );
}


int JParserObj::outputType( int n ) {
  if ( n >= MAXVALS*2 ) return JStringData::id;
  else if ( n >= MAXVALS ) return JRealData::id;
  return JIntegerData::id;
}
int JParserObj::inputType( int n ) {
  if ( n == 0 ) return JStringData::id;
  return JIntegerData::id;
}
JString JParserObj::outputTag(int n) {
  char* tag[] = {
    "intVal_1",
    "intVal_2",
    "intVal_3",
    "intVal_4",
    "intVal_5",
    "intVal_6",
    "intVal_7",
    "intVal_8",
    "intVal_9",
    "intVal_10",
    "floatVal_1",
    "floatVal_2",
    "floatVal_3",
    "floatVal_4",
    "floatVal_5",
    "floatVal_6",
    "floatVal_7",
    "floatVal_8",
    "floatVal_9",
    "floatVal_10",
    "strVal_1",
    "strVal_2",
    "strVal_3",
    "strVal_4",
    "strVal_5",
    "strVal_6",
    "strVal_7",
    "strVal_8",
    "strVal_9",
    "strVal_10" };
  return( tag[n] );
}


JFRect JParserObj::inputArea(int n) {
    return leftArea( n, 0, 1, 0.1, 0.8 );
}


JFRect JParserObj::outputArea(int n) {
  return rightArea(n, 0, MAXVALS + MAXVALS + MAXVALS, 0.1);
}


int JParserObj::inputFace(int n) {
  if ( n < 1 ) {
    return LEFT;
  } else {
    return TOP;
  }
}


int JParserObj::outputFace(int n) {
  return RIGHT;
}


JFPoint JParserObj::inputPoint(int n, JLinkObj& link) {
  return leftPoint( n, link, 0, 1, 0.1, 0.8);
}


JFPoint JParserObj::outputPoint(int n, JLinkObj& link) {
  return rightPoint(n, link, 0, MAXVALS + MAXVALS + MAXVALS, 0.1);
}


void JParserObj::access(int n, JLinkObj& link, const JDataType& data) {
  if ( n < MAXVALS ) {
     INT(data) = intVal[n];
  } else if ( n < MAXVALS * 2 ) {
     DOUBLE(data) = (double)floatVal[n-MAXVALS];
  } else {
	  JSTRING(data) = strVal[n-MAXVALS*2];
  }
}

#include <malloc.h>
#include <string.h>
char* newstrcpy( char* s ) {
	char* result = (char*)malloc( strlen( s ) + 1 );
	strcpy( result, s );
	return( result );
}

#include <stdio.h>
#include <stdlib.h>

void JParserObj::engine(int n, JLinkObj& link) {
	int i = 0;
	int len = format.length();
	static char* savedSvals[] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
char dbuf[100];
  switch (n) {
  case INSTRING:
    link.access( JStringData( inString ));
	if ( inString.length() > len ) {
		int iVal = 0;
		float fVal = 0.0f;
		char sVal[100];
		int sValIdx = 0;
		int strValIdx = 0;
		int floatValIdx = 0;
		sVal[0] = 0;
		int inStrIdx = 0;
		int inStrLen = inString.length();
		for ( i = 0; i < len; i++ ) {
			switch( format[i] ) {
			case 's':
				//JComponent::debug( "Process String" );
				sVal[0] = 0;
				sValIdx = 0;
				while ( inStrIdx <= inStrLen ) {
					char cval = 0;
					if ( inStrIdx < inStrLen ) {
						cval = inString[ inStrIdx ];
					sprintf( dbuf, "process char %c", cval );
					//JComponent::debug( newstrcpy( dbuf ));
					}
					if ((( cval >= '0' ) && ( cval <= '9' )) || ( cval == '.' ) || ( cval == 0 )) {
						if ( sValIdx > 0 ) {
							if ( savedSvals[ strValIdx ] != NULL ) {
								free( savedSvals[ strValIdx ] );
							}
							savedSvals[ strValIdx ] = newstrcpy( &sVal[0] );
							strVal[ strValIdx ] = JString( savedSvals[ strValIdx ]);
							sprintf( dbuf, "broadcasting str %d, val: %s", strValIdx, savedSvals[ strValIdx ] );
							//JComponent::debug( newstrcpy( dbuf ));
							broadcast( MAXVALS * 2 + strValIdx );
							strValIdx++;
							sValIdx = 0;
							sVal[0] = 0;
							break;
						}
					} else {
						sVal[ sValIdx++ ] = cval;
						sVal[ sValIdx ] = 0;
					}
					inStrIdx++;
				}

				break;
			case 'f':
				//JComponent::debug( "Process Float" );
				sValIdx = 0;
				sVal[0] = 0;
				while ( inStrIdx <= inStrLen ) {
					char cval = 0;
					if ( inStrIdx < inStrLen ) {
						cval = inString[ inStrIdx ];
					sprintf( dbuf, "process char %c", cval );
					//JComponent::debug( newstrcpy( dbuf ));
					}
					if ((( cval >= '0' ) && ( cval <= '9' )) || ( cval == '.' )) {
						sVal[ sValIdx++ ] = cval;
						sVal[ sValIdx ] = 0;
						inStrIdx++;
					} else {
						if ( sValIdx > 0 ) {
							floatVal[ floatValIdx ] = (float)atof( sVal );
							broadcast( MAXVALS + floatValIdx );
						}
						floatValIdx++;
						break;
					}
				}
				break;
			case 'd':
				JComponent::debug( "Process Int" );
				break;
			case 'c':
				//JComponent::debug( "Process Char" );
				sValIdx = 0;
				sVal[0] = 0;
				if ( inStrIdx < inStrLen ) {
					char cval = inString[ inStrIdx ];
					sVal[ sValIdx++ ] = cval;
					sVal[ sValIdx ] = 0;
					if ( savedSvals[ strValIdx ] != NULL ) {
						free( savedSvals[ strValIdx ] );
					}
					savedSvals[ strValIdx ] = newstrcpy( &sVal[0] );
					strVal[ strValIdx ] = JString( savedSvals[ strValIdx ]);
					broadcast( MAXVALS * 2 + strValIdx );
					strValIdx++;
					inStrIdx++;
				}
				break;
			default:
				break;
			}
		}
	}
    break;
  default:
    break;
  }
}


JArray JParserObj::getProperties() {
  JArray properties;
  properties.append(JStringProperty("format", format));
  return properties;
}


boolean JParserObj::updateProperty(JProperty& prop) {
  if (prop.getName() == JString("format")) {
    format = ((JStringProperty*)&prop)->value;
    return true;
  }
  return false;
}


